/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.couple;

/**
 * Pair of two type, useful to return two types by a function.
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 * 
 * @param <K>
 * @param <V>
 */
public class MPair<K, V> {
	K first;
	V second;

	/**
	 * Consturctor with the two elements of the pair
	 * @param first
	 * @param second
	 */
	public MPair(K first, V second) {
		this.first = first;
		this.second = second;
	}

	/**
	 * 
	 * @return the first element of the pair
	 */
	public K getFirst() {
		return first;
	}

	/**
	 * 
	 * @return the second element of the pair
	 */
	public V getSecond() {
		return second;
	}

	/**
	 * Set the first element of the pair
	 * @param first
	 */
	public void setFirst(K first) {
		this.first = first;
	}

	/**
	 * Set the second element of the pair
	 * @param first
	 */
	public void setSecond(V second) {
		this.second = second;
	}

	public boolean equals(MPair<K, V> o) {
		return this.first.equals(o.first) && this.second.equals(o.second);
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((first == null) ? 0 : first.hashCode());
		result = prime * result + ((second == null) ? 0 : second.hashCode());
		return result;
	}


	public String toString() {
		return "(" + this.first + " , " + this.second + ")";
	}

}
