/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.matrix;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import fr.ufrt.xmlcorrector.operations.OperationSequence;

/**
 * List of Cells.
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public class Cells {
	ArrayList<Cell> thecells;
	int minimal_cost;

	/**
	 * Constructor that creates a Cells by copying the given Cells
	 * 
	 * @param Cells
	 *            Cell to copy
	 * 
	 */
	public Cells(Cells cell) {

		this.thecells = new ArrayList<Cell>();

		Iterator<Cell> i = cell.thecells.iterator();
		while (i.hasNext())
			this.thecells.add(new Cell(i.next()));

		this.minimal_cost = cell.getMinimalCost();

	}

	/**
	 * Constructor that creates a cells without any OperationSequence but with a
	 * cost
	 * 
	 * @param cost
	 *            Cost of the cell
	 */
	public Cells(int cost) {

		this.thecells = new ArrayList<Cell>();
		this.minimal_cost = cost;

	}

	public Cells(int cost, HashMap<Integer, HashSet<OperationSequence>> hm) {
		this.minimal_cost = cost;
		this.thecells = new ArrayList<Cell>();

		Set<Integer> c = hm.keySet();
		Iterator<Integer> itc = c.iterator();
		while (itc.hasNext()) {
			int k = itc.next();
			this.thecells.add(new Cell(k, hm.get(k)));
		}
	}

	public Cells(Cell c) {
		this.minimal_cost = c.getCost();
		this.thecells = new ArrayList<Cell>();
		this.thecells.add(c);
	}

	/**
	 * Returns the minimal cost of the ArrayList
	 * 
	 * @return minimal cost
	 */
	public int getMinimalCost() {
		return this.minimal_cost;
	}

	/**
	 * Changes the current minimal cost with the cost given
	 * 
	 * @param cost
	 *            Cost to use
	 */
	public void setCost(int cost) {

		this.minimal_cost = cost;

	}

	/**
	 * Returns the ArrayList of cell
	 * 
	 * @return thecells the ArrayList of cell
	 */
	public ArrayList<Cell> getCells() {
		return this.thecells;
	}

	/**
	 * Changes the ArrayList with the ArrayList given
	 * 
	 * @param ArrayList
	 *            <Cell> cells to use
	 */
	public void setCost(ArrayList<Cell> cells) {

		this.thecells = new ArrayList<Cell>(cells);

	}

}
