/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.matrix;

/**
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public abstract class Matrix<T> {

	/**
	 * Gets the width of the Matrix
	 * 
	 * @return Width of the Matrix
	 */
	public abstract int getWidth();

	/**
	 * Gets the height of the Matrix
	 * 
	 * @return Height of the Matrix
	 */
	public abstract int getHeight();

	/**
	 * Returns the value contained by the cell of the Matrix at (x, y)
	 * 
	 * @param x
	 *            abscissa of the cell
	 * @param y
	 *            ordinate of the cell
	 * @return Value of the cell
	 */
	public abstract T getValue(int x, int y);

	/**
	 * 
	 * Changes the value contained by the cell of the Matrix at (x, y), returns
	 * null if the position doesn't exists
	 * 
	 * @param x
	 *            abscissa of the cell
	 * @param y
	 *            ordinate of the cell
	 * @param value
	 */
	public abstract void setValue(int x, int y, T value);

	public String toString() {

		String s = "";
		s = s + "Largeur: " + getWidth() + "\n";
		s = s + "Hauteur: " + getHeight() + "\n";
		s = s + "Valeurs: \n";

		for (int i = 0; i < getHeight(); i++) {

			for (int j = 0; j < getWidth(); j++) {

				s = s + "\t" + getValue(j, i);

			}

			s = s + "\n";

		}

		return s;

	}

}
