/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.matrix;

import java.util.Hashtable;

/**
 * This class Extented a Matrix with a new column.
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public class RightExtendedMatrix<T> extends Matrix<T> {

	Hashtable<Integer, T> values;
	Matrix<T> matrix;
	int delta;

	/**
	 * Constructor for the RightExtendedMatrix, will use a matrix a source and
	 * use it for every value that is in. The given matrix will be used as
	 * reference, so every modification onto it will be done in this one.
	 * 
	 * @param matrix
	 *            Matrix to use as source
	 */
	public RightExtendedMatrix(Matrix<T> matrix) {

		this.matrix = matrix;
		this.delta = this.matrix.getWidth() - 1;
		this.values = new Hashtable<Integer, T>();

	}

	public int getHeight() {

		return this.matrix.getHeight();

	}

	public int getWidth() {

		return this.delta + 2;

	}

	public T getValue(int x, int y) {

		// if we want a value that is in the left side matrix

		if (x <= this.delta) {

			return this.matrix.getValue(x, y);

		}

		// We wan't it from the right side

		else if (x == this.delta + 1) {

			return this.values.get(y);

		}

		// No value ?

		else
			return null;

	}

	public void setValue(int x, int y, T value) {

		// Set a value from the left side

		if (x <= this.delta) {

			this.matrix.setValue(x, y, value);

		}

		// From the right side

		else if (x == this.delta + 1) {

			this.values.put(y, value);

		}

	}

}
