/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.matrix;

import java.util.HashMap;

/**
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public class SimpleMatrix<T> extends Matrix<T> {

	HashMap<String, T> values;
	int height;
	int width;

	/**
	 * Constructor that creates a Matrix with the given height and width
	 * 
	 * @param width
	 *            Width of the Matrix
	 * @param height
	 *            Height of the MAtrix
	 */
	public SimpleMatrix(int width, int height) {

		this.width = width;
		this.height = height;
		values = new HashMap<String, T>();

	}

	/**
	 * Constructor that creates a Matrix with the given height and width. <br />
	 * Then it copies the given value in every Cell
	 * 
	 * @param width
	 *            Width of the Matrix
	 * @param height
	 *            Height of the MAtrix
	 * @param value
	 *            Value that should be put in every cell
	 */
	public SimpleMatrix(int width, int height, T value) {

		// Let's create the Matrix

		this.width = width;
		this.height = height;
		values = new HashMap<String, T>();

		// And put in every Cell the value

		for (int i = 0; i < width; i++) {

			for (int j = 0; j < height; j++) {

				setValue(i, j, value);

			}

		}

	}

	/**
	 * Constructor that copies the given matrix
	 * 
	 * @param matrix
	 *            Matrix to copy
	 */
	public SimpleMatrix(Matrix<T> matrix) {

		this.width = matrix.getWidth();
		this.height = matrix.getHeight();
		values = new HashMap<String, T>();

		// And put in every Cell the value

		for (int i = 0; i < getWidth(); i++) {

			for (int j = 0; j < getHeight(); j++) {

				this.setValue(i, j, matrix.getValue(i, j));

			}

		}

	}

	/**
	 * Creates a matrix with the given length and height and copies the values
	 * of matrix that can fit in
	 * 
	 * @param width
	 *            Width of the Matrix
	 * @param height
	 *            Height of the Matrix
	 * @param matrix
	 *            Matrix that should be copied
	 */
	public SimpleMatrix(int width, int height, Matrix<T> matrix) {

		this.width = width;
		this.height = height;
		values = new HashMap<String, T>();

		// And put in every Cell the value

		for (int i = 0; i < getWidth(); i++) {

			for (int j = 0; j < getHeight(); j++) {

				this.setValue(i, j, matrix.getValue(i, j));

			}

		}

	}

	public T getValue(int x, int y) {

		String key = x + "-" + y;
		return values.get(key);

	}

	public void setValue(int x, int y, T value) {

		String key = x + "-" + y;
		values.put(key, value);

	}

	@Override
	public int getHeight() {
		return this.height;
	}

	@Override
	public int getWidth() {
		return this.width;
	}

}
