/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.operations.edit;

import org.jdom.Element;

import fr.ufrt.xmlcorrector.exceptions.headerNodeException;
import fr.ufrt.xmlcorrector.exceptions.noNodeException;
import fr.ufrt.xmlcorrector.operations.Operation;
import fr.ufrt.xmlcorrector.tree.Address;
import fr.ufrt.xmlcorrector.tree.Tree;
import fr.ufrt.xmlcorrector.vocabtranslator.VocabTranslator;

/**
 * Relabel Node Operation.
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public class RelabelNodeOperation extends EditOperation {

	Character header;

	public static int cost = 1;

	/**
	 * Constructor of the operation RelabelNodeOperation
	 * 
	 * @param address
	 *            The Address of the Node to relabel
	 * @param header
	 *            The header to use to relabel the Node
	 * @throws headerNodeException
	 *             Throws this exception is the header of the new node is ""
	 */
	public RelabelNodeOperation(Address address, Character header)
			throws headerNodeException {

		// Let's verify

		// if(header == "") throw new headerNodeException();

		// It's okay

		this.address = address;
		this.header = header;
		this.operation = OperationType.relabelNode;

	}

	/**
	 * Calculates the cost of the Operation
	 * 
	 * @return Cost of the Operation
	 */
	public int getCost() {

		return 1;

	}

	public int getCost(Tree tree) {

		return 1;

	}

	public void make(Tree tree) throws noNodeException, headerNodeException {

		tree.relabelNode(this.address, this.header);

	}

	public String toString() {

		return "(" + this.operation + ", \"" + this.address + "\", \""
				+ this.header + "\")";

	}

	public Operation clone() {

		try {

			return new RelabelNodeOperation(new Address(this.address),
					this.header);

		}

		catch (headerNodeException e) {

			return null;

		}

	}

	public boolean equals(Object o) {

		if (o instanceof RelabelNodeOperation)
			return o.hashCode() == this.hashCode();
		else
			return false;

	}

	public int hashCode() {

		return (this.operation + "o" + this.getAddress().getAddress() + "o" + this.header)
				.hashCode();

	}

	public Element toXmlElement() {

		Element result = new Element(this.operation.toString());
		result.setAttribute("Address", this.address.toString());
		result.setAttribute("Node", this.header.toString());

		return result;

	}

	/*
	 * @param vt Vocabulary translator establishing the correspondence between
	 * XML elements and characters
	 */
	public Element toXmlElement(VocabTranslator vt) {

		Element result = new Element(this.operation.toString());
		result.setAttribute("Address", this.address.toString());
		result.setAttribute("Node", vt.getElem(this.header));

		return result;

	}

	public static void changeOperationCost(int cost) {

		if (cost >= 0)
			RelabelNodeOperation.cost = cost;

	}

}
