/*
 *   Copyright 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY
 *   <joshua.amavi@gmail.com><beatrice.bouchou@univ-tours.fr><agata.savary@univ-tours.fr>
 *
 *   This file is part of XMLCorrector.
 *
 *   XMLCorrector is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   XMLCorrector is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License,
 *   along with XMLCorrector.  If not, see <http://www.gnu.org/licenses/>.
*/

package fr.ufrt.xmlcorrector.vue;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.PlainDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import org.bounce.text.ScrollableEditorPanel;
import org.bounce.text.xml.XMLEditorKit;
import org.bounce.text.xml.XMLFoldingMargin;
import org.bounce.text.xml.XMLStyleConstants;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import fr.ufrt.xmlcorrector.controller.XmlCorrectorController;

/**
 * Class Fenetre: the Graphical User Interface. 
 * 
 * @author Joshua AMAVI
 * @author Alexandre BOREL
 * @author Béatrice BOUCHOU
 * @author Agata SAVARY
 */
public class Fenetre extends JFrame implements ActionListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = 6858760188048502439L;
	private JPanel programme = new JPanel();
	private JEditorPane statistics = null;
	private JTabbedPane onglet = new JTabbedPane();

	private JPanel lfichiers = new JPanel();
	private JPanel lsequences = new JPanel();
	private JPanel treedisplay = new JPanel();
	private JPanel treecorrectdisplay = new JPanel();
	private JPanel sequencedisplay = new JPanel();
	private JSplitPane ongletResult;

	private JMenuBar menuBar = new JMenuBar();
	private JMenu File = new JMenu("File");
	private JMenu Help = new JMenu("Help");

	//private JMenuItem item1 = new JMenuItem("Open");
	private JMenuItem item2 = new JMenuItem("Options");
	private JMenuItem item3 = new JMenuItem("Exit");
	// private JMenuItem item4 = new JMenuItem("A propos de XmlCorrector");
	private JMenuItem item4 = new JMenuItem("About XmlCorrector");

	private JComboBox threshold = new JComboBox();
	private JCheckBox checkbox1 = new JCheckBox("");
	private JTextField text1 = new JTextField("");
	private JTextField text2 = new JTextField("");
	private JTextField text3 = new JTextField("");
	private JButton bouton = new JButton("Launch");
	private JButton bt1 = new JButton("...");
	private JButton bt2 = new JButton("...");

	private JSplitPane split;

	private int addcost = 1;
	private int relabelcost = 1;
	private int removecost = 1;

	private XmlCorrectorController corrector;
	private String defaultFilePath = null;
	Thread tt;

	/**
	 * Main method...
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new Fenetre();
	}

	private void initMenu() {
		this.setTitle("XmlCorrector");
		//this.File.add(item1);
		this.File.add(item2);
		item2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				XDialog zd = new XDialog(null, "Options", true);
				zd.showXDialog();
				// JOptionPane jop = new JOptionPane();
				// String msg = "addcost " + addcost + "\nrelabelcost " +
				// relabelcost + "\nremovecost " + removecost ;
				// jop.showMessageDialog(null, msg, "Infobulle",
				// JOptionPane.INFORMATION_MESSAGE);

			}
		});
		item3.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
		this.File.add(item3);
		this.Help.add(item4);
		item4.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JEditorPane credits = new JEditorPane();
				credits.setEditable(false);
				credits.setMargin(new Insets(25, 25, 25, 25));
				String msg = "<HTML>\n<HEAD></HEAD>\n<BODY>\n <h2><font color=\"red\">XmlCorrector</font></h2>"
						+ "<font color=gray>version 1.0</font><br/>"
						+ "XMLCorrector is an implementation of an algorithm allowing to correct an XML document with respect to schema constraints expressed as a DTD.<br/><br/>"
						+ "<font color=blue>http://www.info.univ-tours.fr/~savary/English/xmlcorrector.html</font><br/><br/>"
						+ "Copyright &#169; 2012 Joshua AMAVI, Alexandre BOREL, Béatrice BOUCHOU, Agata SAVARY<br/><br/>"
						+ "This program comes with ABSOLUTELY NO WARRANTY; for details visit the website <a href=\"\">http://www.gnu.org/licenses/lgpl.html</a>"
						+ "</HTML>";// Tous droits réservés		
				
				credits.setEditorKit(new HTMLEditorKit());
				credits.setText(msg);
				JDialog jd = new JDialog();
				jd.setSize(500, 350);
				jd.setLocationRelativeTo(null);
				jd.setResizable(false);
				jd.setTitle("About XmlCorrector");
				jd.getContentPane().add(credits, BorderLayout.CENTER);
				jd.setVisible(true);

			}
		});

		this.menuBar.add(File);
		this.menuBar.add(Help);
		this.setJMenuBar(menuBar);

	}

	private void initProgrammePannel() throws IOException {
		// coté Data
		programme.setSize(new Dimension(300, 850));
		programme.setBorder(BorderFactory.createTitledBorder("Data"));

		text1.setPreferredSize(new Dimension(100, 20));
		JPanel ptext1 = new JPanel();
		JLabel ltext1 = new JLabel("Tree Schema");
		ltext1.setPreferredSize(new Dimension(120, 20));
		bt1 = new JButton("...");
		bt1.setPreferredSize(new Dimension(30, 20));
		bt1.addActionListener(this);
		ptext1.add(ltext1);
		ptext1.add(text1);
		ptext1.add(bt1);
		ptext1.setPreferredSize(new Dimension(280, 40));

		text2.setPreferredSize(new Dimension(100, 20));
		JPanel ptext2 = new JPanel();
		JLabel ltext2 = new JLabel("Tree to correct");
		ltext2.setPreferredSize(new Dimension(120, 20));
		bt2 = new JButton("...");
		bt2.setPreferredSize(new Dimension(30, 20));
		bt2.addActionListener(this);
		ptext2.add(ltext2);
		ptext2.add(text2);
		ptext2.add(bt2);
		ptext2.setPreferredSize(new Dimension(280, 40));

		for (int i = 0; i < 50; i++)
			threshold.addItem(i);
		threshold.setPreferredSize(new Dimension(100, 20));
		JLabel lthreshold = new JLabel("Threshold");
		lthreshold.setPreferredSize(new Dimension(120, 20));
		JPanel pthreshold = new JPanel();
		pthreshold.add(lthreshold);
		pthreshold.add(threshold);
		JLabel lthreshold2 = new JLabel("");
		lthreshold2.setPreferredSize(new Dimension(30, 20));
		pthreshold.add(lthreshold2);
		pthreshold.setPreferredSize(new Dimension(280, 40));

		checkbox1.setPreferredSize(new Dimension(100, 20));
		JLabel lcheckbox1 = new JLabel("Minimal");
		lcheckbox1.setPreferredSize(new Dimension(120, 20));
		JLabel lcheckbox2 = new JLabel("");
		lcheckbox2.setPreferredSize(new Dimension(30, 20));
		JPanel pcheckbox1 = new JPanel();
		pcheckbox1.add(lcheckbox1);
		pcheckbox1.add(checkbox1);
		pcheckbox1.add(lcheckbox2);
		pcheckbox1.setPreferredSize(new Dimension(280, 40));

		text3.setPreferredSize(new Dimension(100, 20));
		JPanel ptext3 = new JPanel();
		JLabel ltext3 = new JLabel("Result FileName");
		ltext3.setPreferredSize(new Dimension(120, 20));
		JLabel ltext3a = new JLabel("");
		ltext3a.setPreferredSize(new Dimension(30, 20));
		ptext3.add(ltext3);
		ptext3.add(text3);
		ptext3.add(ltext3a);
		ptext3.setPreferredSize(new Dimension(280, 40));

		bouton.setPreferredSize(new Dimension(100, 20));
		bouton.addActionListener(new CorrectorListener());
		JLabel lbouton = new JLabel("");
		lbouton.setPreferredSize(new Dimension(120, 20));
		JLabel lbouton2 = new JLabel("");
		lbouton2.setPreferredSize(new Dimension(30, 20));
		JPanel pbouton = new JPanel();
		pbouton.add(lbouton);
		pbouton.add(bouton);
		pbouton.add(lbouton2);
		pbouton.setPreferredSize(new Dimension(280, 40));

		programme.add(ptext1);
		programme.add(ptext2);
		programme.add(pthreshold);
		programme.add(pcheckbox1);
		programme.add(ptext3);
		programme.add(pbouton);

		// coté statistics
		statistics = new JEditorPane();

		// Instantiate a XMLEditorKit
		XMLEditorKit kit = new XMLEditorKit();

		statistics.setEditorKit(kit);
		statistics.setEditable(false);

		// Set the font style.
		statistics.setFont(new Font("Courier", Font.PLAIN, 12));

		// Set the tab size
		statistics.getDocument().putProperty(PlainDocument.tabSizeAttribute,
				new Integer(4));

		// Enable auto indentation.
		kit.setAutoIndentation(true);

		// Enable tag completion.
		kit.setTagCompletion(true);

		// Enable error highlighting.
		statistics.getDocument().putProperty(
				XMLEditorKit.ERROR_HIGHLIGHTING_ATTRIBUTE, new Boolean(true));

		// Set a style
		kit.setStyle(XMLStyleConstants.ATTRIBUTE_NAME, new Color(255, 0, 0),
				Font.BOLD);

		// Put the editor in a panel that will force it to resize, when a
		// different
		// view is choosen.
		ScrollableEditorPanel editorPanel = new ScrollableEditorPanel(
				statistics);

		JScrollPane scroller = new JScrollPane(editorPanel);

		// Add the number margin and folding margin as a Row Header View
		JPanel rowHeader = new JPanel(new BorderLayout());
		rowHeader.add(new XMLFoldingMargin(statistics), BorderLayout.EAST);
		// rowHeader.add(new LineNumberMargin(statistics), BorderLayout.WEST);
		scroller.setRowHeaderView(rowHeader);
		// scroller.setSize(new Dimension(450,600));
		scroller.setBorder(BorderFactory.createTitledBorder("Statistics"));

		// Jonction des deux cotés
		split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, programme, scroller);
		split.setDividerLocation(300);
		split.setEnabled(false);

	}

	private void initResultsPannel() {
		lfichiers.setPreferredSize(new Dimension(300, 275));
		lsequences.setPreferredSize(new Dimension(300, 2000));
		sequencedisplay.setPreferredSize(new Dimension(300, 177));

		treedisplay.setPreferredSize(new Dimension(550, 350));
		treecorrectdisplay.setPreferredSize(new Dimension(550, 350));

		JSplitPane sp1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, lfichiers,
				new JScrollPane(lsequences));
		sp1.setDividerLocation(215);
		sp1.setEnabled(false);
		JSplitPane sp2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, treedisplay,
				treecorrectdisplay);
		sp2.setDividerLocation(310);
		sp2.setEnabled(false);
		JSplitPane sp3 = new JSplitPane(JSplitPane.VERTICAL_SPLIT, sp1,
				sequencedisplay);
		sp3.setDividerLocation(430);
		sp3.setEnabled(false);

		ongletResult = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, sp3, sp2);
		ongletResult.setDividerLocation(300);
		ongletResult.setEnabled(false);
		ongletResult.updateUI();

	}

	public Fenetre() {
		try {
			this.setSize(850, 700);
			this.setResizable(false);
			initMenu();

			initProgrammePannel();

			initResultsPannel();

			onglet.addTab("Program", split);
			onglet.addTab("Results", ongletResult);

			this.getContentPane().setLayout(new BorderLayout());
			this.getContentPane().add(onglet, BorderLayout.CENTER);

			this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			this.setLocationRelativeTo(null);

			// testVariables();

			this.setVisible(true);

		} catch (Throwable e) {
			e.printStackTrace();
			System.exit(1);
		}
	}

	/** Traite un événement (clic sur le bouton "Choose a file...") */
	@SuppressWarnings("static-access")
	public void actionPerformed(ActionEvent e) {
		JFileChooser jfc = new JFileChooser();
		jfc.addChoosableFileFilter(new XmlFilter());
		jfc.setAcceptAllFileFilterUsed(false);
		if (defaultFilePath != null)
			jfc.setCurrentDirectory(new File(defaultFilePath));

		switch (jfc.showOpenDialog(this)) {
		case JFileChooser.CANCEL_OPTION:
			;
			break;
		case JFileChooser.APPROVE_OPTION:
			if (e.getSource() == bt1) {
				text1.setText(jfc.getSelectedFile().getPath());
				defaultFilePath = jfc.getSelectedFile().getParent();
			} else if (e.getSource() == bt2) {
				text2.setText(jfc.getSelectedFile().getPath());
				defaultFilePath = jfc.getSelectedFile().getParent();
			}
			break;
		case JFileChooser.ERROR_OPTION:
			(new JOptionPane())
					.showMessageDialog(null, "An error has occurred", "Error",
							JOptionPane.ERROR_MESSAGE);
			break;
		default:
			;
		}
	}

	class XDialog extends JDialog {
		/**
		 * 
		 */
		private static final long serialVersionUID = 3617784343080743021L;
		// private boolean sendData;
		private JLabel addLabel, relabelLabel, removeLabel;
		private JTextField addNodeCost, relabelNodeCost, removeNodeCost;

		/**
		 * Constructeur
		 * 
		 * @param parent
		 * @param title
		 * @param modal
		 */
		public XDialog(JFrame parent, String title, boolean modal) {
			super(parent, title, modal);
			this.setSize(300, 250);
			this.setLocationRelativeTo(null);
			this.setResizable(false);
			this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
			this.initComponent();
		}

		/**
		 * Méthode appelée pour utiliser la boîte
		 */
		public void showXDialog() {
			// this.sendData = false;
			this.setVisible(true);
		}

		/**
		 * Initialise le contenu de la boîte
		 */
		private void initComponent() {
			// AddNodeCost
			JPanel panAddNode = new JPanel();
			//panAddNode.setBackground(Color.white);
			panAddNode.setPreferredSize(new Dimension(250, 40));
			addNodeCost = new JTextField();
			addNodeCost.setPreferredSize(new Dimension(50, 30));
			addNodeCost.setText(String.valueOf(addcost));
			addLabel = new JLabel("AddNodeCost : ");
			addLabel.setPreferredSize(new Dimension(160, 30));
			panAddNode.add(addLabel);
			panAddNode.add(addNodeCost);

			// RelabelNodeCost
			JPanel panRelabelNode = new JPanel();
			//panRelabelNode.setBackground(Color.white);
			panRelabelNode.setPreferredSize(new Dimension(250, 40));
			relabelNodeCost = new JTextField();
			relabelNodeCost.setPreferredSize(new Dimension(50, 30));
			relabelNodeCost.setText(String.valueOf(relabelcost));
			relabelLabel = new JLabel("RelabelNodeCost : ");
			relabelLabel.setPreferredSize(new Dimension(160, 30));
			panRelabelNode.add(relabelLabel);
			panRelabelNode.add(relabelNodeCost);

			// RemoveNodeCost
			JPanel panRemoveNode = new JPanel();
			//panRemoveNode.setBackground(Color.white);
			panRemoveNode.setPreferredSize(new Dimension(250, 40));
			removeNodeCost = new JTextField();
			removeNodeCost.setPreferredSize(new Dimension(50, 30));
			removeNodeCost.setText(String.valueOf(removecost));
			removeLabel = new JLabel("RemoveNodeCost : ");
			removeLabel.setPreferredSize(new Dimension(160, 30));
			panRemoveNode.add(removeLabel);
			panRemoveNode.add(removeNodeCost);

			JPanel content = new JPanel();
			//content.setBackground(Color.white);
			content.add(panAddNode);
			content.add(panRelabelNode);
			content.add(panRemoveNode);

			JPanel control = new JPanel();
			JButton okBouton = new JButton("OK");

			okBouton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					addcost = addNodeCost.getText().equals("") ? 0 : Integer
							.valueOf(addNodeCost.getText());
					relabelcost = relabelNodeCost.getText().equals("") ? 0
							: Integer.valueOf(relabelNodeCost.getText());
					removecost = removeNodeCost.getText().equals("") ? 0
							: Integer.valueOf(removeNodeCost.getText());
					setVisible(false);
				}
			});

			JButton cancelBouton = new JButton("Annuler");
			cancelBouton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					setVisible(false);
				}
			});

			control.add(okBouton);
			control.add(cancelBouton);

			this.getContentPane().add(content, BorderLayout.CENTER);
			this.getContentPane().add(control, BorderLayout.SOUTH);

		}
	}

	class CorrectorListener implements ActionListener {
		private String sTreeSchema = null;
		private String sTree = null;
		private int sThreshold = -1;
		private boolean sMinimal = false;
		private String sResultFile = null;
		private int sAddcost = -1;
		private int sRelabelcost = -1;
		private int sRemovecost = -1;
		private JTree arbreT = null;
		private JTree arbreS = null;

		public CorrectorListener() {
			sTreeSchema = null;
			sTree = null;
			sThreshold = -1;
			sMinimal = false;
			sResultFile = null;
			sAddcost = -1;
			sRelabelcost = -1;
			sRemovecost = -1;
		}

		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e) {
			tt = new Thread() {
				public void run() {
					clearPannels();
					ProgressBar progressBar = new ProgressBar(tt);
					// progressBar.createAndShowGUI();
					String msg_error = checkData();
					String msg_error2 = "";
					if (msg_error.equals("")) {
						corrector = new XmlCorrectorController();
						corrector.addObserver(progressBar);

						progressBar.createAndShowGUI();
						msg_error2 = corrector.correction(sTreeSchema, sTree,
								sThreshold, sMinimal, sResultFile, sAddcost,
								sRelabelcost, sRemovecost);

						if (msg_error2.equals("")) {
							progressBar.closeGUI();
							try {
								File file = new File("results/" + sResultFile
										+ ".xml");
								statistics.read(new FileReader(file), file);
								updateResultsPannel("results/" + sResultFile
										+ ".xml");
								updateTreeDisplayPannel();
							} catch (FileNotFoundException err) {
								// TODO Auto-generated catch block
								err.printStackTrace();
							} catch (IOException err) {
								// TODO Auto-generated catch block
								err.printStackTrace();
							}
						} else {
							progressBar.closeGUI();
							JOptionPane jop = new JOptionPane();
							jop.showMessageDialog(null, msg_error2, "Error",
									JOptionPane.ERROR_MESSAGE);
						}
					} else {
						JOptionPane jop = new JOptionPane();
						jop.showMessageDialog(null, msg_error, "Error",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			};
			tt.start();

		}

		private void clearPannels() {
			statistics.setText("");
			lfichiers.removeAll();
			treedisplay.removeAll();
			lsequences.removeAll();
			treecorrectdisplay.removeAll();
			sequencedisplay.removeAll();
			onglet.updateUI();
		}

		private void clearPannels2() {
			lsequences.removeAll();
			treecorrectdisplay.removeAll();
			sequencedisplay.removeAll();
			onglet.updateUI();
		}

		private String checkData() {
			String msg_error = "";

			if (!text1.getText().equals(""))
				sTreeSchema = text1.getText();
			else
				msg_error += "- Select a file for Tree Schema\n";

			if (!text2.getText().equals(""))
				sTree = text2.getText();
			else
				msg_error += "- Select a file for Tree to correct\n";

			sMinimal = checkbox1.isSelected();

			sThreshold = (Integer) threshold.getSelectedItem();
			if (!sMinimal && sThreshold == 0)
				msg_error += "- Threshold cannot be equals to 0\n";

			if (!text3.getText().equals(""))
				sResultFile = text3.getText();
			else
				msg_error += "- Enter a Result Filename\n";

			if (addcost > 0)
				sAddcost = addcost;
			else
				msg_error += "- AddNodeCost must be a positive\n"; // doît être
																	// un
																	// positif
																	// non nul

			if (relabelcost > 0)
				sRelabelcost = relabelcost;
			else
				msg_error += "- RelabelNodeCost must be a positive\n";

			if (removecost > 0)
				sRemovecost = removecost;
			else
				msg_error += "- RemoveNodeCost must be a positive\n";

			return msg_error;
		}

		@SuppressWarnings("unchecked")
		private void updateResultsPannel(String filename) {
			org.jdom.Document document;
			Element e;
			SAXBuilder sxb = new SAXBuilder();

			// Création d'une racine
			DefaultMutableTreeNode racine = new DefaultMutableTreeNode("Files");

			try {
				document = sxb.build(filename);
				Element root = document.getRootElement();

				// Si il y a bien une racine
				if (root != null && root.getName().equals("ResultSet")) {
					e = root.getChild("ResultFiles");

					if (e != null) {
						Iterator<Element> j = e.getChildren().iterator();
						while (j.hasNext()) {
							Element f = j.next();
							if (f.getName().equals("ResultFile")) {
								DefaultMutableTreeNode rep = new DefaultMutableTreeNode(
										f.getText());
								racine.add(rep);
							}
						}
					}
				}

				// On crée, avec notre hiérarchie, un arbre
				arbreT = new JTree(racine);
				arbreT.setPreferredSize(lfichiers.getPreferredSize());
				arbreT.getSelectionModel().setSelectionMode(
						TreeSelectionModel.SINGLE_TREE_SELECTION);

				// Listen for when the selection changes.
				arbreT.addTreeSelectionListener(new TreeSelectionListener() {

					public void valueChanged(TreeSelectionEvent e) {
						// Returns the last path element of the selection.
						// This method is useful only when the selection model
						// allows a single selection.
						DefaultMutableTreeNode node = (DefaultMutableTreeNode) arbreT
								.getLastSelectedPathComponent();

						if (node == null)
							return; // Nothing is selected.
						else {
							String nodeInfo = (String) node.getUserObject();
							clearPannels2();
							updateFilePannel(nodeInfo);
						}

					}
				});

				lfichiers.removeAll();
				lfichiers.add(new JScrollPane(arbreT));

			} catch (JDOMException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

		}

		private void updateTreeDisplayPannel() {
			org.jdom.Document document;
			SAXBuilder sxb = new SAXBuilder();

			try {
				document = sxb.build(text2.getText());
				Element root = document.getRootElement();

				String xmltree = treetoString(root);
				System.out.println(xmltree);
				TreeWithContours twc = new TreeWithContours(
						(new Parser(xmltree)).readTree());
				twc.rel2abs(0);
				twc.symposition();

				TreeDrawing td = new TreeDrawing(twc,
						treedisplay.getPreferredSize().width - 50,
						treedisplay.getPreferredSize().height - 50);

				treedisplay.removeAll();
				treedisplay.setLayout(new BorderLayout());
				treedisplay.add(td, BorderLayout.CENTER);

			} catch (JDOMException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}

		@SuppressWarnings({ "unchecked" })
		private String treetoString(Element e) {
			String res = "(" + e.getName();
			if (!e.getChildren().isEmpty()) {
				Iterator<Element> j = e.getChildren().iterator();
				while (j.hasNext()) {
					Element f = j.next();
					res += treetoString(f);
				}
			}

			res += ")";
			return res;
		}

		private String treetoString(String s) {
			s = s.replace('[', '<').replace(']', '>');

			Matcher m1 = Pattern.compile("<([a-zA-Z0-9_-]*)>").matcher(s);
			s = m1.replaceAll("($1");
			Matcher m2 = Pattern.compile("</([a-zA-Z0-9_-]*)>").matcher(s);
			s = m2.replaceAll(")");
			Matcher m3 = Pattern.compile("<([a-zA-Z0-9_-]*) />").matcher(s);
			s = m3.replaceAll("($1)");

			return s;

		}

		@SuppressWarnings("unchecked")
		private void updateFilePannel(String filename) {
			org.jdom.Document document;
			Element e;
			SAXBuilder sxb = new SAXBuilder();

			// Création d'une racine
			DefaultMutableTreeNode racine = new DefaultMutableTreeNode(
					"Sequences");

			try {
				document = sxb.build("results/" + filename);
				Element root = document.getRootElement();
				int i = 0;
				// Si il y a bien une racine
				if (root != null && root.getName().equals("Result")) {
					e = root.getChild("UpdateOperationSequences");
					DefaultMutableTreeNode reptype = new DefaultMutableTreeNode(
							"UpdateOperation");
					if (e != null) {
						Iterator<Element> j = e.getChildren().iterator();
						while (j.hasNext()) {
							Element f = j.next();
							if (f.getName().equals("OperationSequence")) {
								DefaultMutableTreeNode rep = new DefaultMutableTreeNode(
										"Sequence n° 1." + (++i));
								// System.out.println("Contenu : " +
								// f.getText());
								reptype.add(rep);
							}
						}
					}
					racine.add(reptype);

					e = root.getChild("EditOperationSequences");
					i = 0;
					reptype = new DefaultMutableTreeNode("EditOperation");
					if (e != null) {
						Iterator<Element> j = e.getChildren().iterator();
						while (j.hasNext()) {
							Element f = j.next();
							if (f.getName().equals("OperationSequence")) {
								DefaultMutableTreeNode rep = new DefaultMutableTreeNode(
										"Sequence n° 2." + (++i));
								reptype.add(rep);
							}
						}
					}
					racine.add(reptype);

				}

				// On crée, avec notre hiérarchie, un arbre
				arbreS = new JTree(racine);
				arbreS.setName(filename);

				arbreS.setPreferredSize(lsequences.getPreferredSize());
				arbreS.getSelectionModel().setSelectionMode(
						TreeSelectionModel.SINGLE_TREE_SELECTION);

				// Listen for when the selection changes.
				arbreS.addTreeSelectionListener(new TreeSelectionListener() {

					public void valueChanged(TreeSelectionEvent e) {
						// Returns the last path element of the selection.
						// This method is useful only when the selection model
						// allows a single selection.
						DefaultMutableTreeNode node = (DefaultMutableTreeNode) arbreS
								.getLastSelectedPathComponent();

						if (node == null)
							return; // Nothing is selected.
						else {
							String nodeInfo = (String) node.getUserObject();

							System.out.println(nodeInfo);
							updateSequenceAndCorrecttreePannels(nodeInfo,
									arbreS.getName());
						}
					}
				});

				lsequences.removeAll();
				lsequences.add(arbreS);
				onglet.updateUI();

			} catch (JDOMException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

		}

		@SuppressWarnings("unchecked")
		private void updateSequenceAndCorrecttreePannels(String s,
				String filename) {
			if (!s.equals("EditOperation") && !s.equals("UpdateOperation")
					&& !s.equals("Sequences")) {
				String position = s.substring(12);

				int parent = Integer.valueOf(position.substring(0,
						position.indexOf('.')));
				int child = Integer.valueOf(position.substring(position
						.indexOf('.') + 1));

				org.jdom.Document document;
				Element e = null;
				SAXBuilder sxb = new SAXBuilder();

				String correcttree = "";
				String sequences = "";

				try {
					document = sxb.build("results/" + filename);
					Element root = document.getRootElement();
					if (parent == 1)
						e = (Element) root.getChild("UpdateOperationSequences")
								.getChildren().get(child - 1);
					else if (parent == 2)
						e = (Element) root.getChild("EditOperationSequences")
								.getChildren().get(child - 1);

					if (e != null) {
						Iterator<Element> j = e.getChildren().iterator();
						while (j.hasNext()) {
							Element f = j.next();
							if (f.getName().equals("Tree"))
								correcttree = f.getText();
							else {
								String adresse = " Address=\""
										+ f.getAttributeValue("Address") + "\"";
								String tree = f.getAttributeValue("Tree") == null ? ""
										: " Tree=\""
												+ f.getAttributeValue("Tree")
												+ "\"";
								String node = f.getAttributeValue("Node") == null ? ""
										: " Node=\""
												+ f.getAttributeValue("Node")
												+ "\"";
								sequences += "<" + f.getName() + adresse + tree
										+ node + "/>\n";
							}

						}

						// Mise à jour de sequencedisplay
						// System.out.println(sequences);
						updateSequencePannel(sequences);

						// Mise à jour de treecorrectdisplay
						updateCorrectTree(correcttree);
						System.out.println(correcttree);

						// onglet.repaint();
						onglet.updateUI();

					}

				} catch (JDOMException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}

			}
		}

		private void updateCorrectTree(String tree) {
			System.out.println(tree);

			String xmltree = treetoString(tree);
			System.out.println("xmltree " + xmltree);

			TreeWithContours twc = new TreeWithContours(
					(new Parser(xmltree)).readTree());
			twc.rel2abs(0);
			twc.symposition();

			TreeDrawing td = new TreeDrawing(twc,
					treecorrectdisplay.getPreferredSize().width - 50,
					treecorrectdisplay.getPreferredSize().height - 50);

			treecorrectdisplay.removeAll();
			treecorrectdisplay.setLayout(new BorderLayout());
			treecorrectdisplay.add(td, BorderLayout.CENTER);

			// treecorrectdisplay.repaint();
			// onglet.repaint();

		}

		private void updateSequencePannel(String s) {
			JEditorPane jep = new JEditorPane();

			// Instantiate a XMLEditorKit
			XMLEditorKit kit = new XMLEditorKit();

			jep.setEditorKit(kit);
			jep.setEditable(false);

			// Set the font style.
			jep.setFont(new Font("Courier", Font.PLAIN, 12));

			// Set the tab size
			jep.getDocument().putProperty(PlainDocument.tabSizeAttribute,
					new Integer(4));

			// Enable auto indentation.
			kit.setAutoIndentation(true);

			// Enable tag completion.
			kit.setTagCompletion(true);

			// Enable error highlighting.
			jep.getDocument().putProperty(
					XMLEditorKit.ERROR_HIGHLIGHTING_ATTRIBUTE,
					new Boolean(true));

			// Set a style
			kit.setStyle(XMLStyleConstants.ATTRIBUTE_NAME,
					new Color(255, 0, 0), Font.BOLD);

			// Put the editor in a panel that will force it to resize, when a
			// different
			// view is choosen.
			ScrollableEditorPanel editorPanel = new ScrollableEditorPanel(jep);

			JScrollPane scroller = new JScrollPane(editorPanel);

			// Add the number margin and folding margin as a Row Header View
			// JPanel rowHeader = new JPanel(new BorderLayout());
			// rowHeader.add(new XMLFoldingMargin(jep), BorderLayout.EAST);
			// rowHeader.add(new LineNumberMargin(statistics),
			// BorderLayout.WEST);
			// scroller.setRowHeaderView(rowHeader);
			scroller.setPreferredSize(sequencedisplay.getPreferredSize());

			jep.setText(s);
			sequencedisplay.removeAll();
			sequencedisplay.add(scroller);

		}
	}

}
