package fr.ufrt.xmlcorrector.vue;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import javax.swing.JPanel;

public class TreeDrawing extends JPanel {
	private static final long serialVersionUID = 1L;
	private DrawableTreeNode tree;
	private int xsteps, xoffset, ysteps, yoffset;

	public TreeDrawing(DrawableTreeNode t, int width, int height) {
		tree = t;
		xsteps = (int) (width / (t.getMaxX() - t.getMinX() + 1));
		ysteps = (int) (height / (t.getHeight() + 1));
		xoffset = (int) (xsteps * (0.5 - t.getMinX()));
		yoffset = ysteps / 2;

	}

	public int scale(double f) {
		return (int) (f * xsteps) + xoffset;
	}

	public int scale(int f) {
		return (f * ysteps) + yoffset;
	}

	public void drawLine(Graphics g, DrawableTreeNode p1, DrawableTreeNode p2,
			int y) {
		g.drawLine(scale(p1.getX()), scale(y), scale(p2.getX()), scale(y + 1));
	}

	public void paint(Graphics g, DrawableTreeNode p, int ord) {
		String name = p.getLabel();
		int x = scale(p.getX());
		int y = scale(ord);
		FontMetrics m = g.getFontMetrics();
		int rad = m.getAscent();
		int len = m.stringWidth(name);
		g.setColor(Color.lightGray);
		g.fillOval(x - len / 2 - rad, y - rad, len + 2 * rad, 2 * rad);
		g.setColor(Color.black);
		g.drawOval(x - len / 2 - rad, y - rad, len + 2 * rad, 2 * rad);
		g.drawString(name, x - len / 2, y + (m.getAscent() - m.getDescent())
				/ 2);
	}

	public void paintTree(Graphics g, DrawableTreeNode t, int y) {
		for (DrawableTreeNode pp : t.getChildren()) {
			drawLine(g, t, pp, y);
			paintTree(g, pp, y + 1);
		}
		paint(g, t, y);
	}

	/*
	 * public void paint(Graphics g) { paintTree(g, tree, 0); }
	 */

	public void paintComponent(Graphics g) {
		paintTree(g, tree, 0);
	}

}
